package sun.util.locale.provider;

import java.security.PrivilegedAction;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import java.io.File;
import java.util.Collections;
import java.util.StringTokenizer;
import java.util.HashSet;
import java.util.spi.LocaleServiceProvider;
import java.util.concurrent.ConcurrentHashMap;
import sun.util.spi.CalendarProvider;
import java.util.spi.CalendarNameProvider;
import java.util.spi.CalendarDataProvider;
import java.util.spi.TimeZoneNameProvider;
import java.util.spi.LocaleNameProvider;
import java.util.spi.CurrencyNameProvider;
import java.text.spi.NumberFormatProvider;
import java.text.spi.DecimalFormatSymbolsProvider;
import java.text.spi.DateFormatSymbolsProvider;
import java.text.spi.DateFormatProvider;
import java.text.spi.CollatorProvider;
import java.text.spi.BreakIteratorProvider;
import sun.util.resources.LocaleData;
import java.util.Locale;
import java.util.Set;
import java.util.concurrent.ConcurrentMap;

public class JRELocaleProviderAdapter extends LocaleProviderAdapter implements ResourceBundleBasedAdapter
{
    private static final String LOCALE_DATA_JAR_NAME = "localedata.jar";
    private final ConcurrentMap<String, Set<String>> langtagSets;
    private final ConcurrentMap<Locale, LocaleResources> localeResourcesMap;
    private volatile LocaleData localeData;
    private volatile BreakIteratorProvider breakIteratorProvider;
    private volatile CollatorProvider collatorProvider;
    private volatile DateFormatProvider dateFormatProvider;
    private volatile DateFormatSymbolsProvider dateFormatSymbolsProvider;
    private volatile DecimalFormatSymbolsProvider decimalFormatSymbolsProvider;
    private volatile NumberFormatProvider numberFormatProvider;
    private volatile CurrencyNameProvider currencyNameProvider;
    private volatile LocaleNameProvider localeNameProvider;
    private volatile TimeZoneNameProvider timeZoneNameProvider;
    private volatile CalendarDataProvider calendarDataProvider;
    private volatile CalendarNameProvider calendarNameProvider;
    private volatile CalendarProvider calendarProvider;
    private static volatile Boolean isNonENSupported;
    
    public JRELocaleProviderAdapter() {
        this.langtagSets = new ConcurrentHashMap();
        this.localeResourcesMap = new ConcurrentHashMap();
        this.breakIteratorProvider = null;
        this.collatorProvider = null;
        this.dateFormatProvider = null;
        this.dateFormatSymbolsProvider = null;
        this.decimalFormatSymbolsProvider = null;
        this.numberFormatProvider = null;
        this.currencyNameProvider = null;
        this.localeNameProvider = null;
        this.timeZoneNameProvider = null;
        this.calendarDataProvider = null;
        this.calendarNameProvider = null;
        this.calendarProvider = null;
    }
    
    @Override
    public LocaleProviderAdapter.Type getAdapterType() {
        return LocaleProviderAdapter.Type.JRE;
    }
    
    @Override
    public <P extends LocaleServiceProvider> P getLocaleServiceProvider(Class<P> clazz) {
        String simpleName = clazz.getSimpleName();
        int n = -1;
        switch (simpleName.hashCode()) {
            case 907948926: {
                if (simpleName.equals("BreakIteratorProvider")) {
                    n = 0;
                    break;
                }
                break;
            }
            case 90734195: {
                if (simpleName.equals("CollatorProvider")) {
                    n = 1;
                    break;
                }
                break;
            }
            case -125048202: {
                if (simpleName.equals("DateFormatProvider")) {
                    n = 2;
                    break;
                }
                break;
            }
            case -1614968633: {
                if (simpleName.equals("DateFormatSymbolsProvider")) {
                    n = 3;
                    break;
                }
                break;
            }
            case 38915396: {
                if (simpleName.equals("DecimalFormatSymbolsProvider")) {
                    n = 4;
                    break;
                }
                break;
            }
            case -1353096495: {
                if (simpleName.equals("NumberFormatProvider")) {
                    n = 5;
                    break;
                }
                break;
            }
            case -798486419: {
                if (simpleName.equals("CurrencyNameProvider")) {
                    n = 6;
                    break;
                }
                break;
            }
            case 231772470: {
                if (simpleName.equals("LocaleNameProvider")) {
                    n = 7;
                    break;
                }
                break;
            }
            case 1502810229: {
                if (simpleName.equals("TimeZoneNameProvider")) {
                    n = 8;
                    break;
                }
                break;
            }
            case 1028302393: {
                if (simpleName.equals("CalendarDataProvider")) {
                    n = 9;
                    break;
                }
                break;
            }
            case 759873562: {
                if (simpleName.equals("CalendarNameProvider")) {
                    n = 10;
                    break;
                }
                break;
            }
            case 1334414191: {
                if (simpleName.equals("CalendarProvider")) {
                    n = 11;
                    break;
                }
                break;
            }
        }
        switch (n) {
            case 0: {
                return this.getBreakIteratorProvider();
            }
            case 1: {
                return this.getCollatorProvider();
            }
            case 2: {
                return this.getDateFormatProvider();
            }
            case 3: {
                return this.getDateFormatSymbolsProvider();
            }
            case 4: {
                return this.getDecimalFormatSymbolsProvider();
            }
            case 5: {
                return this.getNumberFormatProvider();
            }
            case 6: {
                return this.getCurrencyNameProvider();
            }
            case 7: {
                return this.getLocaleNameProvider();
            }
            case 8: {
                return this.getTimeZoneNameProvider();
            }
            case 9: {
                return this.getCalendarDataProvider();
            }
            case 10: {
                return this.getCalendarNameProvider();
            }
            case 11: {
                return this.getCalendarProvider();
            }
            default: {
                throw new InternalError("should not come down here");
            }
        }
    }
    
    @Override
    public BreakIteratorProvider getBreakIteratorProvider() {
        if (this.breakIteratorProvider == null) {
            BreakIteratorProviderImpl breakIteratorProvider = new BreakIteratorProviderImpl(this.getAdapterType(), this.getLanguageTagSet("FormatData"));
            synchronized (this) {
                if (this.breakIteratorProvider == null) {
                    this.breakIteratorProvider = breakIteratorProvider;
                }
            }
        }
        return this.breakIteratorProvider;
    }
    
    @Override
    public CollatorProvider getCollatorProvider() {
        if (this.collatorProvider == null) {
            CollatorProviderImpl collatorProvider = new CollatorProviderImpl(this.getAdapterType(), this.getLanguageTagSet("CollationData"));
            synchronized (this) {
                if (this.collatorProvider == null) {
                    this.collatorProvider = collatorProvider;
                }
            }
        }
        return this.collatorProvider;
    }
    
    @Override
    public DateFormatProvider getDateFormatProvider() {
        if (this.dateFormatProvider == null) {
            DateFormatProviderImpl dateFormatProvider = new DateFormatProviderImpl(this.getAdapterType(), this.getLanguageTagSet("FormatData"));
            synchronized (this) {
                if (this.dateFormatProvider == null) {
                    this.dateFormatProvider = dateFormatProvider;
                }
            }
        }
        return this.dateFormatProvider;
    }
    
    @Override
    public DateFormatSymbolsProvider getDateFormatSymbolsProvider() {
        if (this.dateFormatSymbolsProvider == null) {
            DateFormatSymbolsProviderImpl dateFormatSymbolsProvider = new DateFormatSymbolsProviderImpl(this.getAdapterType(), this.getLanguageTagSet("FormatData"));
            synchronized (this) {
                if (this.dateFormatSymbolsProvider == null) {
                    this.dateFormatSymbolsProvider = dateFormatSymbolsProvider;
                }
            }
        }
        return this.dateFormatSymbolsProvider;
    }
    
    @Override
    public DecimalFormatSymbolsProvider getDecimalFormatSymbolsProvider() {
        if (this.decimalFormatSymbolsProvider == null) {
            DecimalFormatSymbolsProviderImpl decimalFormatSymbolsProvider = new DecimalFormatSymbolsProviderImpl(this.getAdapterType(), this.getLanguageTagSet("FormatData"));
            synchronized (this) {
                if (this.decimalFormatSymbolsProvider == null) {
                    this.decimalFormatSymbolsProvider = decimalFormatSymbolsProvider;
                }
            }
        }
        return this.decimalFormatSymbolsProvider;
    }
    
    @Override
    public NumberFormatProvider getNumberFormatProvider() {
        if (this.numberFormatProvider == null) {
            NumberFormatProviderImpl numberFormatProvider = new NumberFormatProviderImpl(this.getAdapterType(), this.getLanguageTagSet("FormatData"));
            synchronized (this) {
                if (this.numberFormatProvider == null) {
                    this.numberFormatProvider = numberFormatProvider;
                }
            }
        }
        return this.numberFormatProvider;
    }
    
    @Override
    public CurrencyNameProvider getCurrencyNameProvider() {
        if (this.currencyNameProvider == null) {
            CurrencyNameProviderImpl currencyNameProvider = new CurrencyNameProviderImpl(this.getAdapterType(), this.getLanguageTagSet("CurrencyNames"));
            synchronized (this) {
                if (this.currencyNameProvider == null) {
                    this.currencyNameProvider = currencyNameProvider;
                }
            }
        }
        return this.currencyNameProvider;
    }
    
    @Override
    public LocaleNameProvider getLocaleNameProvider() {
        if (this.localeNameProvider == null) {
            LocaleNameProviderImpl localeNameProvider = new LocaleNameProviderImpl(this.getAdapterType(), this.getLanguageTagSet("LocaleNames"));
            synchronized (this) {
                if (this.localeNameProvider == null) {
                    this.localeNameProvider = localeNameProvider;
                }
            }
        }
        return this.localeNameProvider;
    }
    
    @Override
    public TimeZoneNameProvider getTimeZoneNameProvider() {
        if (this.timeZoneNameProvider == null) {
            TimeZoneNameProviderImpl timeZoneNameProvider = new TimeZoneNameProviderImpl(this.getAdapterType(), this.getLanguageTagSet("TimeZoneNames"));
            synchronized (this) {
                if (this.timeZoneNameProvider == null) {
                    this.timeZoneNameProvider = timeZoneNameProvider;
                }
            }
        }
        return this.timeZoneNameProvider;
    }
    
    @Override
    public CalendarDataProvider getCalendarDataProvider() {
        if (this.calendarDataProvider == null) {
            CalendarDataProviderImpl calendarDataProvider = new CalendarDataProviderImpl(this.getAdapterType(), this.getLanguageTagSet("CalendarData"));
            synchronized (this) {
                if (this.calendarDataProvider == null) {
                    this.calendarDataProvider = calendarDataProvider;
                }
            }
        }
        return this.calendarDataProvider;
    }
    
    @Override
    public CalendarNameProvider getCalendarNameProvider() {
        if (this.calendarNameProvider == null) {
            CalendarNameProviderImpl calendarNameProvider = new CalendarNameProviderImpl(this.getAdapterType(), this.getLanguageTagSet("FormatData"));
            synchronized (this) {
                if (this.calendarNameProvider == null) {
                    this.calendarNameProvider = calendarNameProvider;
                }
            }
        }
        return this.calendarNameProvider;
    }
    
    @Override
    public CalendarProvider getCalendarProvider() {
        if (this.calendarProvider == null) {
            CalendarProviderImpl calendarProvider = new CalendarProviderImpl(this.getAdapterType(), this.getLanguageTagSet("CalendarData"));
            synchronized (this) {
                if (this.calendarProvider == null) {
                    this.calendarProvider = calendarProvider;
                }
            }
        }
        return this.calendarProvider;
    }
    
    @Override
    public LocaleResources getLocaleResources(Locale locale) {
        LocaleResources localeResources = (LocaleResources)this.localeResourcesMap.get(locale);
        if (localeResources == null) {
            localeResources = new LocaleResources(this, locale);
            LocaleResources localeResources2 = (LocaleResources)this.localeResourcesMap.putIfAbsent(locale, localeResources);
            if (localeResources2 != null) {
                localeResources = localeResources2;
            }
        }
        return localeResources;
    }
    
    @Override
    public LocaleData getLocaleData() {
        if (this.localeData == null) {
            synchronized (this) {
                if (this.localeData == null) {
                    this.localeData = new LocaleData(this.getAdapterType());
                }
            }
        }
        return this.localeData;
    }
    
    @Override
    public Locale[] getAvailableLocales() {
        return (Locale[])JRELocaleProviderAdapter.AvailableJRELocales.localeList.clone();
    }
    
    public Set<String> getLanguageTagSet(String s) {
        Set<String> languageTagSet = (Set)this.langtagSets.get(s);
        if (languageTagSet == null) {
            languageTagSet = this.createLanguageTagSet(s);
            Set<Object> set = (Set)this.langtagSets.putIfAbsent(s, languageTagSet);
            if (set != null) {
                languageTagSet = set;
            }
        }
        return languageTagSet;
    }
    
    protected Set<String> createLanguageTagSet(String s) {
        String supportedLocaleString = LocaleDataMetaInfo.getSupportedLocaleString(s);
        if (supportedLocaleString == null) {
            return Collections.emptySet();
        }
        HashSet<String> set = new HashSet();
        StringTokenizer stringTokenizer = new StringTokenizer(supportedLocaleString);
        while (stringTokenizer.hasMoreTokens()) {
            String nextToken = stringTokenizer.nextToken();
            if (nextToken.equals("|")) {
                if (isNonENLangSupported()) {
                    continue;
                }
                break;
            }
            else {
                set.add(nextToken);
            }
        }
        return set;
    }
    
    private static Locale[] createAvailableLocales() {
        String supportedLocaleString = LocaleDataMetaInfo.getSupportedLocaleString("AvailableLocales");
        if (supportedLocaleString.length() == 0) {
            throw new InternalError("No available locales for JRE");
        }
        int index = supportedLocaleString.indexOf(124);
        StringTokenizer stringTokenizer;
        if (isNonENLangSupported()) {
            stringTokenizer = new StringTokenizer(supportedLocaleString.substring(0, index) + supportedLocaleString.substring(index + 1));
        }
        else {
            stringTokenizer = new StringTokenizer(supportedLocaleString.substring(0, index));
        }
        int countTokens = stringTokenizer.countTokens();
        Locale[] array = new Locale[countTokens + 1];
        array[0] = Locale.ROOT;
        for (int i = 1; i <= countTokens; ++i) {
            String nextToken = stringTokenizer.nextToken();
            int n = -1;
            switch (nextToken.hashCode()) {
                case 1601894167: {
                    if (nextToken.equals("ja-JP-JP")) {
                        n = 0;
                        break;
                    }
                    break;
                }
                case -472985013: {
                    if (nextToken.equals("no-NO-NY")) {
                        n = 1;
                        break;
                    }
                    break;
                }
                case -518283308: {
                    if (nextToken.equals("th-TH-TH")) {
                        n = 2;
                        break;
                    }
                    break;
                }
            }
            switch (n) {
                case 0: {
                    array[i] = JRELocaleConstants.JA_JP_JP;
                    break;
                }
                case 1: {
                    array[i] = JRELocaleConstants.NO_NO_NY;
                    break;
                }
                case 2: {
                    array[i] = JRELocaleConstants.TH_TH_TH;
                    break;
                }
                default: {
                    array[i] = Locale.forLanguageTag(nextToken);
                    break;
                }
            }
        }
        return array;
    }
    
    private static boolean isNonENLangSupported() {
        if (JRELocaleProviderAdapter.isNonENSupported == null) {
            synchronized (JRELocaleProviderAdapter.class) {
                if (JRELocaleProviderAdapter.isNonENSupported == null) {
                    String separator = File.separator;
                    JRELocaleProviderAdapter.isNonENSupported = (Boolean)AccessController.doPrivileged(new PrivilegedAction<Boolean>() {
                        
                        @Override
                        public Boolean run() {
                            return Boolean.valueOf(f.exists());
                        }
                    });
                }
            }
        }
        return (boolean)JRELocaleProviderAdapter.isNonENSupported;
    }
    
    static Locale[] access$100() {
        return createAvailableLocales();
    }
    
    static {
        JRELocaleProviderAdapter.isNonENSupported = null;
    }
    
    private static class AvailableJRELocales
    {
        private static final Locale[] localeList;
        
        static {
            localeList = JRELocaleProviderAdapter.access$100();
        }
    }
}
